from .translation import *
from .core import (
    State,
    Action,
    LoginAction,
    CaptureArticleAction,
    TranslateArticleAction,
    NavigateToArticleAction,
    ActionBranch
)


workflow = Action(
    LoginAction(),
    ActionBranch(
        NavigateToArticleAction(),
        CaptureArticleAction(),
        TranslateArticleAction(),
        read_key="article_ids",
        write_key="article_id",
    )
)



import typing as T
import logging

from .clients.base import BaseTranslationClient

logger = logging.getLogger()


class TranslationManager(object):
    clients: T.List[BaseTranslationClient]

    def __init__(
        self
    ) -> None:
        self.clients = []

    def add_clients(
        self,
        *clients: BaseTranslationClient
    ):
        self.clients.extend(clients)

    def get_supported_clients(
        self,
        from_language: dict,
        to_language: dict,
    ) -> T.List[BaseTranslationClient]:
        return [
            client for client in self.clients if client.supports_translation(
                from_language=from_language,
                to_language=to_language
            )
        ]

    def translate(
        self,
        text: str,
        from_language: dict,
        to_language: dict
    ):
        for client in self.get_supported_clients(
            from_language=from_language,
            to_language=to_language
        ):
            try:
                translated_text = client.translate(
                    text=text,
                    from_language=from_language,
                    to_language=to_language
                )
                return translated_text
            except Exception as e:
                logger.exception(e)
                continue

        raise Exception("No clients were able to complete the translation")

    def translate_batch(
        self,
        texts: T.List[str],
        from_language: dict,
        to_language: dict
    ):
        for client in self.get_supported_clients(
            from_language=from_language,
            to_language=to_language
        ):
            try:
                translated_text = client.translate_batch(
                    texts=texts,
                    from_language=from_language,
                    to_language=to_language
                )
                return translated_text
            except Exception as e:
                logger.exception(e)
                continue

        raise Exception("No clients were able to complete the translation")



import time
import logging
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys

from .translation.helpers import translate, translateHTML

logger = logging.getLogger()


class State(object):
    def __init__(self, **kwargs) -> None:
        for k, v in kwargs.items():
            self.__dict__[k] = v

    def __getattr__(self, key: str):
        return self.__dict__.get(key, None)

    def __setattr__(self, key: str, value: any):
        self.__dict__[key] = value

    def __getitem__(self, key: str):
        return self.__getattr__(key)

    def __setitem__(self, key: str, value: any):
        self.__setattr__(key, value)

    def get(self, key: str):
        return self.__getitem__(key)

    def set(self, key: str, value: any):
        self.__setitem__(key, value)

    def mutate(self, **kwargs):
        return State(
            **{
                **self.__dict__,
                **kwargs
            }
        )


class Action(object):
    def __init__(
        self,
        *actions: "Action"
    ) -> None:
        self.actions = actions

    def execute(self, driver: webdriver.Chrome, state: State):
        for action in self.actions:
            action.execute(driver, state)


class ActionBranch(Action):
    def __init__(
        self,
        *actions: "Action",
        read_key: str,
        write_key: str
    ) -> None:
        super().__init__(*actions)
        self.read_key = read_key
        self.write_key = write_key

    def execute(
        self,
        driver: webdriver.Chrome,
        state: State
    ):
        for item in state[self.read_key]:
            mutated_state = state.mutate(
                **{
                    self.write_key: item
                }
            )
            super().execute(driver, mutated_state)


class SkipPopupAction(Action):
    def execute(self, driver: webdriver.Chrome, state: State):
        for data_key in [
            'knowledgeContentUI.edit.newInsertMenuPopover.dismissButton',
            'introModal.buttons.gotIt'
        ]:
            try:
                label = driver.find_element(By.CSS_SELECTOR, f'[data-key="{data_key}"]')
                if not label:
                    return
                button = label.find_element(By.XPATH, "./..")
                if not button:
                    return
                button.click()
                time.sleep(1)
            except Exception:
                pass


class LoginAction(Action):
    def execute(self, driver: webdriver.Chrome, state: State):
        driver.get('https://app.hubspot.com/login/beta')

        username_input = driver.find_element(value="username")
        username_input.send_keys(state.email)

        login_button = driver.find_element(value="loginBtn")
        login_button.click()
        time.sleep(1)

        password_input = driver.find_element(value="current-password")
        password_input.send_keys(state.password)

        login_button = driver.find_element(value="loginBtn")
        login_button.click()
        time.sleep(1)

        code_input = driver.find_element(value="code")
        code_input.send_keys(state.code)

        login_button = driver.find_element(
            by=By.CLASS_NAME,
            value="login-submit"
        )
        login_button.click()
        time.sleep(1)

        remember_button = driver.find_element(
            by=By.CSS_SELECTOR,
            value='[data-2fa-rememberme="false"]'
        )
        if remember_button:
            remember_button.click()
        time.sleep(3)


class NavigateToArticleAction(Action):
    def execute(self, driver: webdriver.Chrome, state: State):
        driver.get(f"https://app.hubspot.com/knowledge/{state.knowledge_base_id}/edit/{state.article_id}")
        time.sleep(3)
        SkipPopupAction().execute(driver, state)


class CreateVariantArticleAction(Action):
    def execute(self, driver: webdriver.Chrome, state: State):
        language_name = state.to_languages[state.to_language]
        create_variation_label = driver.find_element(By.CSS_SELECTOR, '[data-key="knowledgeContentUI.edit.languageSwitcher.languageActions.createLanguageVariant"]')
        create_variation_button = create_variation_label.find_element(By.XPATH, "./..")
        create_variation_button.click()
        time.sleep(1)

        language_dropdown_button = driver.find_element(By.CLASS_NAME, 'private-dropdown__caret--form')
        language_dropdown_button.click()
        time.sleep(1)

        language_selection_button = driver.find_element(By.CSS_SELECTOR, f'[title="{language_name}"]')
        if not language_selection_button:
            logger.warning(f"Unable to create `{language_name}` variant for Article `{state.article_id}`... Looks like the language isn't setup in Hubspot")
            return False

        language_selection_div = language_selection_button.find_element(By.XPATH, "./../..")
        if 'is-disabled' in language_selection_div.get_attribute('class'):
            logger.warning(f"Unable to create `{language_name}` variant for Article `{state.article_id}`... apparently is already exists but I couldn't find it")
            return False

        language_selection_button.click()
        time.sleep(1)

        save_label = driver.find_element(By.CSS_SELECTOR, '[data-key="knowledgeContentUI.manage.createLanguageVariantModal.buttons.save"]')
        save_button = save_label.find_element(By.XPATH, "./..")
        save_button.click()
        time.sleep(1)

        driver.refresh()
        time.sleep(3)
        return True


class NavigateToVariantArticleAction(Action):
    def execute(self, driver: webdriver.Chrome, state: State):
        language_name = state.to_languages[state.to_language]

        header = driver.find_element(By.CSS_SELECTOR, '[aria-label="Page Section"]')
        language_popup_button = header.find_elements(By.TAG_NAME, "BUTTON")[1]
        language_popup_button.click()
        time.sleep(1)

        language_popup_div = driver.find_element(By.CSS_SELECTOR, '[role="presentation"]')
        languages_tbody = language_popup_div.find_element(By.TAG_NAME, "TBODY")
        if language_name not in languages_tbody.text:
            if not CreateVariantArticleAction().execute(driver, state):
                return False

            return self.execute(driver, state)

        language_buttons = languages_tbody.get_property('children')
        language_button = [btn for btn in language_buttons if language_name in btn.text]
        if not language_button:
            return False

        language_button = language_button[0]
        if "Draft" not in language_button.text:
            logger.info(f"`{language_name}` variant of Article `{state.article_id}` is not in draft state, skipping...")
            return False

        language_button.click()
        time.sleep(1)

        SkipPopupAction().execute(driver, state)
        return True


class ArticleAction(Action):
    def get_title(self, driver: webdriver.Chrome):
        title_textarea = driver.find_element(By.CLASS_NAME, 'article-title-text')
        return title_textarea.text

    def get_subtitle(self, driver: webdriver.Chrome):
        title_textarea = driver.find_element(By.CLASS_NAME, 'article-subtitle-text')
        return title_textarea.text

    def get_body(self, driver: webdriver.Chrome):
        title_textarea = driver.find_element(By.CLASS_NAME, 'mce-content-body')
        return title_textarea.get_attribute("innerHTML")

    def set_title(self, driver: webdriver.Chrome, value):
        element = driver.find_element(By.CLASS_NAME, 'article-title-text')
        element.clear()
        element.send_keys(value)

    def set_subtitle(self, driver: webdriver.Chrome, value):
        element = driver.find_element(By.CLASS_NAME, 'article-subtitle-text')
        element.clear()
        element.send_keys(value)

    def set_body(self, driver: webdriver.Chrome, value):
        element = driver.find_element(By.CLASS_NAME, 'mce-content-body')
        driver.execute_script(
            'arguments[0].innerHTML = arguments[1]',
            element,
            value
        )
        self.save(driver, element)

    def save(self, driver: webdriver.Chrome, element):
        time.sleep(0.2)
        element.send_keys(" ")
        time.sleep(0.2)
        element.send_keys(Keys.BACKSPACE)
        time.sleep(0.2)


class CaptureArticleAction(ArticleAction):
    def execute(self, driver: webdriver.Chrome, state: State):
        state.title = self.get_title(driver)
        state.subtitle = self.get_subtitle(driver)
        state.body = self.get_body(driver)


class TranslateArticleAction(ArticleAction):
    def execute(self, driver: webdriver.Chrome, state: State):
        for state.to_language in state.to_languages:
            if NavigateToVariantArticleAction().execute(driver, state):
                self.set_title(
                    driver,
                    translate(
                        text=state.title,
                        from_language=state.from_language,
                        to_language=state.to_language
                    )
                )

                self.set_subtitle(
                    driver,
                    translate(
                        text=state.subtitle,
                        from_language=state.from_language,
                        to_language=state.to_language
                    )
                )

                self.set_body(
                    driver,
                    translateHTML(
                        html=state.body,
                        from_language=state.from_language,
                        to_language=state.to_language
                    )
                )
                SkipPopupAction().execute(driver, state)
                time.sleep(2)

                PublishArticleAction().execute(driver, state)
            else:
                time.sleep(3)
                driver.execute_script("window.onbeforeunload = function() {};")
                driver.refresh()
                time.sleep(3)


class PublishArticleAction(Action):
    def execute(self, driver: webdriver.Chrome, state: State):
        if not state.auto_publish:
            return

        publish_label = driver.find_element(By.CSS_SELECTOR, f'[data-key="knowledgeContentUI.button.publish"]')
        if not publish_label:
            logger.error(f"Unable to publish article `{state.article_id}... Publish label not found")
            return
        publish_button = publish_label.find_element(By.XPATH, "./..")
        if not publish_button:
            logger.error(f"Unable to publish article `{state.article_id}... Publish button not found")
            return
        publish_button.click()
        time.sleep(6)

        driver.back()
        time.sleep(3)



import json
from pathlib import Path

content_languages: dict[str, dict[str, str]] = {}
with open(Path(__file__).parent / "content_languages.json", "r") as file:
    content_languages = json.load(file)


def get_content_language(code: str):
    return content_languages.get(code)

def set_content_language(code: str, content_langauge: dict[str, str]):
    content_languages[code] = content_langauge



import typing as T

from concurrent.futures import ThreadPoolExecutor, as_completed


class BaseTranslationClient(object):

    def __init__(
        self,
        content_language_key: str
    ) -> None:
        self.content_language_key = content_language_key

    def _raise_not_supported_exception(
        self,
        from_language: dict,
        to_language: dict
    ):
        raise Exception(f"{from_language['code']} or {to_language['code']} is not supported")

    def supports_translation(
        self,
        from_language: dict,
        to_language: dict,
        raise_exception: bool = False
    ) -> bool:
        if not (
            from_language.get(self.content_language_key) and
            to_language.get(self.content_language_key)
        ):
            if raise_exception:
                self._raise_not_supported_exception(
                    from_language=from_language,
                    to_language=to_language
                )
            return False
        return True

    def translate(
        self,
        text: str,
        from_language: dict,
        to_language: dict
    ):
        if not text:
            return ""

        self.supports_translation(
            from_language=from_language,
            to_language=to_language,
            raise_exception=True
        )

        return self._translate(
            text=text,
            from_language_code=from_language[self.content_language_key],
            to_language_code=to_language[self.content_language_key]
        )

    def translate_batch(
        self,
        texts: T.List[str],
        from_language: dict,
        to_language: dict
    ):
        self.supports_translation(
            from_language=from_language,
            to_language=to_language,
            raise_exception=True
        )

        try:
            return self._translate_batch(
                texts=texts,
                from_language_code=from_language[self.content_language_key],
                to_language_code=to_language[self.content_language_key]
            )
        except NotImplementedError:
            pass

        # default optimised batch processing
        translations = []
        with ThreadPoolExecutor(max_workers=4) as executor:
            for result in executor.map(
                self._translate,
                texts,
                [from_language[self.content_language_key]] * len(texts),
                [to_language[self.content_language_key]] * len(texts),
            ):
                translations.append(result)
        return translations

    def _translate(
        self,
        text: str,
        from_language_code: str,
        to_language_code: str
    ):
        raise NotImplementedError()

    def _translate_batch(
        self,
        texts: T.List[str],
        from_language_code: str,
        to_language_code: str
    ):
        raise NotImplementedError()



import typing as T
import uuid
import requests
from .base import BaseTranslationClient


class AzureTranslateClient(BaseTranslationClient):
    key: str
    endpoint: str
    location: str
    api_version: str

    def __init__(self, key: str, location: str, endpoint: str, api_version: str = None, **kwargs) -> None:
        super().__init__(
            content_language_key='azure_code'
        )
        self.key = key
        self.location = location
        self.endpoint = endpoint
        self.api_version = api_version or '3.0'

    def _translate(
        self,
        text: str,
        from_language_code: str,
        to_language_code: str
    ):
        if not text:
            return ""

        translations = self._call_api(
            texts=[text],
            from_language=from_language_code,
            to_languages=[to_language_code]
        )
        return translations[0].get(to_language_code)

    def _translate_batch(
        self,
        texts: T.List[str],
        from_language_code: str,
        to_language_code: str
    ):
        translations = self._call_api(
            texts=texts,
            from_language=from_language_code,
            to_languages=[to_language_code]
        )
        return [translation.get(to_language_code) for translation in translations]

    def _call_api(
        self,
        texts: T.List[str],
        from_language: str,
        to_languages: T.List[str]
    ):
        params = {
            'api-version': self.api_version,
            'from': from_language,
            'to': to_languages
        }

        headers = {
            'Ocp-Apim-Subscription-Key': self.key,
            # location required if you're using a multi-service or regional (not global) resource.
            'Ocp-Apim-Subscription-Region': self.location,
            'Content-type': 'application/json',
            'X-ClientTraceId': str(uuid.uuid4())
        }

        body = [{
            'text': text
        } for text in texts]

        request = requests.post(
            self.endpoint,
            params=params,
            headers=headers,
            json=body
        )
        request.raise_for_status()

        response = request.json()
        translations = [{lang["to"]: lang["text"] for lang in translation["translations"]} for translation in response]

        return translations



from bs4 import BeautifulSoup, NavigableString

from ..content_languages import get_content_language


def translate(
    text: str,
    from_language: str,
    to_language: str
):
    from .. import translation_manager
    return translation_manager.translate(
        text,
        get_content_language(from_language),
        get_content_language(to_language)
    )

def _translateHTML(
    node,
    from_language: str,
    to_language: str
):
    if isinstance(node, NavigableString):
        key = node.string.strip()
        value = node.string
        if key:
            # TODO: Implement your translate function
            value = value.replace(key, translate(key, from_language, to_language))
            node.replace_with(value)
    else:
        for child in node.children:
            _translateHTML(
                child,
                from_language,
                to_language
            )

def translateHTML(
    html: str,
    from_language: str,
    to_language: str
):
    soup = BeautifulSoup(html, "html.parser")
    
    for node in soup.children:
        _translateHTML(
            node,
            from_language,
            to_language
        )

    return str(soup.prettify())



from boto3.session import Session

from .base import BaseTranslationClient


class AWSTranslateClient(BaseTranslationClient):
    client = None

    def __init__(self, aws_session: Session, **kwargs) -> None:
        super().__init__(
            content_language_key='aws_code'
        )
        self.client = aws_session.client('translate')

    def _translate(
        self,
        text: str,
        from_language_code: str,
        to_language_code: str
    ):
        if not text:
            return ""

        response = self.client.translate_text(
            Text=text,
            SourceLanguageCode=from_language_code,
            TargetLanguageCode=to_language_code,
        )

        if (
            not response or
            not response.get('TranslatedText')
        ):
            raise Exception("Unable to translate text")

        return response['TranslatedText']
